@@ -3,20 +3,26 @@ __maintainer__ = 'Niklas Wulff 31.12.2019'
 __contributors__ = 'Fabia Miorelli, Parth Butte'
 __email__ = 'Niklas.Wulff@dlr.de'
 __birthdate__ = '31.12.2019'
-__status__ = 'prod'  # options are: dev, test, prod
+__status__ = 'dev'  # options are: dev, test, prod
+
+
+#----- imports & packages ------
+if __package__ is None or __package__ == '':
+    import sys
+    from os import path
+    sys.path.append(path.dirname(path.dirname(path.dirname(__file__))))
+
 
 import pprint
 import pandas as pd
 import numpy as np
 import warnings
 from pathlib import Path
-import yaml
 from zipfile import ZipFile
 
 
 class DataParser:
-    def __init__(self, parseConfig: dict, globalConfig: dict, localPathConfig: dict, datasetID: str='MiD17',
-                 loadEncrypted=True):
+    def __init__(self, configDict: dict, datasetID: str, loadEncrypted=False):
         """
         Basic class for parsing a mobility survey trip data set. Currently the both German travel surveys MiD 2008 and
         MiD 2017 are pre-configured and one of the two can be given (default: MiD 2017).
@@ -29,16 +35,15 @@ class DataParser:
         parseConfig. For some columns, raw data is transferred to human readable strings and respective columns are
         added. Pandas timestamp columns are synthesized from the given trip start and trip end time information.
 
-        :param parseConfig: A yaml config file holding a dictionary with the keys 'pathRelative'
-        :param globalConfig: A yaml config file holding mainly filenames and file labels
-        :param localPathConfig: A config thats not synchronized holding local paths
+        :param configDict: A dictionary containing multiple yaml config files
         :param datasetID: Currently, MiD08 and MiD17 are implemented as travel survey data sets
         :param loadEncrypted: If True, load an encrypted ZIP file as specified in parseConfig
         """
-        self.datasetID = self.checkDatasetID(datasetID, parseConfig)
-        self.parseConfig = parseConfig
-        self.globalConfig = globalConfig
-        self.rawDataPath = Path(localPathConfig['pathAbsolute'][self.datasetID]) / globalConfig['files'][self.datasetID]['tripsDataRaw']
+        self.parseConfig = configDict['parseConfig']
+        self.globalConfig = configDict['globalConfig']
+        self.localPathConfig = configDict['localPathConfig']
+        self.datasetID = self.checkDatasetID(datasetID, self.parseConfig)
+        self.rawDataPath = Path(self.localPathConfig['pathAbsolute'][self.datasetID]) / self.globalConfig['files'][self.datasetID]['tripsDataRaw']
         self.subDict = {}
         self.rawData = None
         self.data = None
@@ -52,19 +57,11 @@ class DataParser:
                   f"{self.globalConfig['pathAbsolute']['encryptedZipfile']}")
             self.loadEncryptedData(pathToZip=Path(self.globalConfig['pathAbsolute']['encryptedZipfile']) /
                                              self.globalConfig['files'][self.datasetID]['encryptedZipFileB2'],
-                                   pathInZip=globalConfig['files'][self.datasetID]['tripDataZipFileRaw'])
+                                   pathInZip=self.globalConfig['files'][self.datasetID]['tripDataZipFileRaw'])
         else:
             print(f"Starting to retrieve local data file from {self.rawDataPath}")
             self.loadData()
-        self.selectColumns()
-        self.harmonizeVariables()
-        self.convertTypes()
-        self.checkFilterDict()
-        self.filter()
-        self.filterConsistentHours()
-        self.addStrColumns()
-        self.composeStartAndEndTimestamps()
-        print('Parsing completed')
+
 
     def updateFilterDict(self) -> None:
         """
@@ -72,8 +69,8 @@ class DataParser:
 
         :return: None
         """
-        self.__filterDict = self.parseConfig['filterDicts'][self.datasetID]
-        self.__filterDict = {iKey: iVal for iKey, iVal in self.__filterDict.items() if self.__filterDict[iKey] is not
+        self.__filterDict[self.datasetID] = self.parseConfig['filterDicts'][self.datasetID]
+        self.__filterDict[self.datasetID] = {iKey: iVal for iKey, iVal in self.__filterDict[self.datasetID].items() if self.__filterDict[self.datasetID][iKey] is not
                              None}
 
     def checkDatasetID(self, datasetID: str, parseConfig: dict) -> str:
@@ -203,11 +200,11 @@ class DataParser:
             listIndex = dictRaw['datasetID'].index(datasetID)
             return {val[listIndex]: key for (key, val) in dictRaw.items()}
         else:
-            raise ValueError(f'Data set {datasetID} not specified in MiD variable dictionary.')
+            raise ValueError(f'Data set {datasetID} not specified in parseConfig variable dictionary.')
 
     def convertTypes(self):
         """
-        Convert raw column types to predefined python types as specified in parseConfig['inputDTypes']. This is mainly
+        Convert raw column types to predefined python types as specified in parseConfig['inputDTypes'][datasetID]. This is mainly
         done for performance reasons. But also in order to avoid index values that are of type int to be cast to float.
         The function operates only on self.data and writes back changes to self.data
 
@@ -215,11 +212,12 @@ class DataParser:
         """
 
         # Filter for dataset specific columns
-        conversionDict = self.parseConfig['inputDTypes']
+        conversionDict = self.parseConfig['inputDTypes'][self.datasetID]
         keys = {iCol for iCol in conversionDict.keys() if iCol in self.data.columns}
         self.subDict = {key: conversionDict[key] for key in conversionDict.keys() & keys}
         self.data = self.data.astype(self.subDict)
 
+
     def returnDictBottomValues(self, baseDict: dict, lst: list = []) -> list:
         """
         Returns a list of all dictionary values of the last dictionary level (the bottom) of baseDict. The parameter
@@ -245,7 +243,7 @@ class DataParser:
         :return: None
         """
 
-        assert all(isinstance(val, list) for val in self.returnDictBottomValues(self.__filterDict)), \
+        assert all(isinstance(val, list) for val in self.returnDictBottomValues(self.__filterDict[self.datasetID])), \
             f'All values in filter dictionaries have to be lists, but are not'
 
     def returnDictBottomKeys(self, baseDict: dict, lst: list = None) -> list:
@@ -277,14 +275,14 @@ class DataParser:
         :return: None
         """
 
-        print(f'Starting filtering, applying {len(self.returnDictBottomKeys(self.__filterDict))} filters.')
+        print(f'Starting filtering, applying {len(self.returnDictBottomKeys(self.__filterDict[self.datasetID]))} filters.')
         ret = pd.DataFrame(index=self.data.index)
         # Future releases: as discussed before we could indeed work here with a plug and pray approach.
         #  we would need to introduce a filter manager and a folder structure where to look for filters.
         #  this is very similar code than the one from ioproc. If we want to go down this route we should
         #  take inspiration from the code there. It was not easy to get it right in the first place. This
         #  might be easy to code but hard to implement correctly.
-        for iKey, iVal in self.__filterDict.items():
+        for iKey, iVal in self.__filterDict[self.datasetID].items():
             if iKey == 'include':
                 ret = ret.join(self.setIncludeFilter(iVal, self.data.index))
             elif iKey == 'exclude':
@@ -294,7 +292,7 @@ class DataParser:
             elif iKey == 'smallerThan':
                 ret = ret.join(self.setSmallerThanFilter(iVal, self.data.index))
             else:
-                warnings.warn(f'A filter dictionary was defined in the parseConfig with an unknown filtering key. '
+                warnings.warn(f'A filter dictionary was defined in the parseConfig with an unknown filtering key.'
                               f'Current filtering keys comprise include, exclude, smallerThan and greaterThan.'
                               f'Continuing with ignoring the dictionary {iKey}')
         self.data = self.data[ret.all(axis='columns')]
@@ -382,11 +380,31 @@ class DataParser:
 
         :return: No returns, operates only on the class instance
         """
-        dat = self.data
-        self.data = dat.loc[(dat['tripStartClock'] <= dat['tripEndClock']) | (dat['tripEndNextDay'] == 1), :]
-        # If we want to get rid of tripStartClock and tripEndClock (they are redundant variables)
-        # self.data = dat.loc[pd.to_datetime(dat.loc[:, 'tripStartHour']) <= pd.to_datetime(dat.loc[:, 'tripEndHour']) |
-        #                     (dat['tripEndNextDay'] == 1), :]
+
+        if self.datasetID == 'MiD17' or self.datasetID == 'MiD08':
+            dat = self.data
+            self.data = dat.loc[(dat['tripStartClock'] <= dat['tripEndClock']) | (dat['tripEndNextDay'] == 1), :]
+            # If we want to get rid of tripStartClock and tripEndClock (they are redundant variables)
+            # self.data = dat.loc[pd.to_datetime(dat.loc[:, 'tripStartHour']) <= pd.to_datetime(dat.loc[:,
+            # 'tripEndHour']) | (dat['tripEndNextDay'] == 1), :]
+            filter = (self.data.loc[:, 'tripStartHour'] == self.data.loc[:, 'tripEndHour']) & \
+                     (self.data.loc[:, 'tripStartMinute'] == self.data.loc[:, 'tripEndMinute']) & \
+                     (self.data.loc[:, 'tripEndNextDay'])
+
+            self.data = self.data.loc[~filter, :]
+
+
+    def addStrColumnFromVariable(self, colName: str, varName: str):
+        """
+        Replaces each occurence of a MiD/KiD variable e.g. 1,2,...,7 for weekdays with an explicitly mapped string e.g.
+        'MON', 'TUE',...,'SUN'.
+
+        :param colName: Name of the column in self.data where the explicit string info is stored
+        :param varName: Name of the VencoPy internal variable given in config/parseConfig['dataVariables']
+        :return: None
+        """
+        self.data.loc[:, colName] \
+            = self.data.loc[:, varName].replace(self.parseConfig['Replacements'][self.datasetID][varName])
 
     def addStrColumns(self, weekday=True, purpose=True):
         """
@@ -398,21 +416,10 @@ class DataParser:
         """
 
         if weekday:
-            self.addStrColumnFromMidVariable(colName='weekdayStr', varName='tripStartWeekday')
+            self.addStrColumnFromVariable(colName='weekdayStr', varName='tripStartWeekday')
         if purpose:
-            self.addStrColumnFromMidVariable(colName='purposeStr', varName='tripPurpose')
+            self.addStrColumnFromVariable(colName='purposeStr', varName='tripPurpose')
 
-    def addStrColumnFromMidVariable(self, colName: str, varName: str):
-        """
-        Replaces each occurence of a MID variable e.g. 1,2,...,7 for weekdays with an explicitly mapped string e.g.
-        'MON', 'TUE',...,'SUN'.
-
-        :param colName: Name of the column in self.data where the explicit string info is stored
-        :param varName: Name of the VencoPy internal variable given in config/parseConfig['dataVariables']
-        :return: None
-        """
-        self.data.loc[:, colName] \
-            = self.data.loc[:, varName].replace(self.parseConfig['midReplacements'][varName])
 
     def composeTimestamp(self, data: pd.DataFrame = None,
                          colYear: str = 'tripStartYear',
@@ -447,7 +454,6 @@ class DataParser:
                               colHour='tripEndHour',
                               colMin='tripEndMinute',
                               colName='timestampEnd')
-        self.updateEndTimestamp()
 
     def updateEndTimestamp(self):
         """
@@ -455,27 +461,153 @@ class DataParser:
         """
         endsFollowingDay = self.data['tripEndNextDay'] == 1
         self.data.loc[endsFollowingDay, 'timestampEnd'] = self.data.loc[endsFollowingDay,
-                                                                        'timestampEnd'] + pd.offsets.Day(1)
+                                                                    'timestampEnd'] + pd.offsets.Day(1)
 
 
-class ParseMID(DataParser):
+    def updateEndTimestamps(self) -> np.datetime64:
+        """
+        :return: Returns start and end time of a trip
+        """
+        self.updateEndTimestamp()
+
+    def harmonizeVariablesGenericIdNames(self):
+        """
+
+        """
+        self.data['genericID'] = self.data[str(self.parseConfig['IDVariablesNames'][self.datasetID])]
+        print('Finished harmonization of ID variables')
+
+    def process(self):
+        """
+        Wrapper function for harmonising and filtering the dataset.
+        """
+        self.selectColumns()
+        self.harmonizeVariables()
+        self.convertTypes()
+        self.checkFilterDict()
+        self.filter()
+        self.filterConsistentHours()
+        self.addStrColumns()
+        self.composeStartAndEndTimestamps()
+        self.updateEndTimestamps()
+        self.harmonizeVariablesGenericIdNames()
+        print('Parsing completed')
+
+
+
+class ParseMiD(DataParser):
     # Inherited data class to differentiate between abstract interfaces such as vencopy internal
     # variable namings and data set specific functions such as filters etc. Currently not used (06/14/2021)
     pass
 
+class ParseKiD(DataParser):
+    # Inherited data class to differentiate between abstract interfaces such as vencopy internal
+    # variable namings and data set specific functions such as filters etc.
+    def __init__(self, configDict: dict, datasetID: str):
+        super().__init__(configDict, datasetID)
 
 
+    def loadData(self):
+        rawDataPathTrips = Path(configDict['localPathConfig']['pathAbsolute'][self.datasetID]) / configDict['globalConfig']['files'][self.datasetID]['tripsDataRaw']
+        rawDataPathVehicles = Path(configDict['localPathConfig']['pathAbsolute'][self.datasetID]) / configDict['globalConfig']['files'][self.datasetID]['vehiclesDataRaw']
+        rawDataTrips = pd.read_stata(rawDataPathTrips, convert_categoricals=False, convert_dates=False,
+                                     preserve_dtypes=False)
+        rawDataVehicles = pd.read_stata(rawDataPathVehicles, convert_categoricals=False, convert_dates=False,
+                                        preserve_dtypes=False)
+        rawDataVehicles.set_index('k00', inplace=True)
+        rawData = rawDataTrips.join(rawDataVehicles, on='k00')
+        self.rawData = rawData
+        print(f'Finished loading {len(self.rawData)} rows of raw data of type .dta')
+
+
+    def addStrColumns(self, weekday=True, purpose=True):
+        """
+        Adds string columns for either weekday or purpose.
+
+        :param weekday: Boolean identifier if weekday string info should be added in a separate column
+        :param purpose: Boolean identifier if purpose string info should be added in a separate column
+        :return: None
+        """
+
+        # from tripStartDate retrieve tripStartWeekday, tripStartWeek, tripStartYear, tripStartMonth, tripStartDay
+        # from tripStartClock retrieve tripStartHour, tripStartMinute
+        # from tripEndClock retrieve tripEndHour, tripEndMinute
+        self.data['tripStartDate'] = pd.to_datetime(self.data['tripStartDate'], format='%d.%m.%Y')
+        self.data['tripStartYear'] = self.data['tripStartDate'].dt.year
+        self.data['tripStartMonth'] = self.data['tripStartDate'].dt.month
+        self.data['tripStartDay'] = self.data['tripStartDate'].dt.day
+        self.data['tripStartWeekday'] = self.data['tripStartDate'].dt.weekday
+        self.data['tripStartWeek'] = self.data['tripStartDate'].dt.isocalendar().week
+        self.data['tripStartHour'] = pd.to_datetime(self.data['tripStartClock'], format='%H:%M').dt.hour
+        self.data['tripStartMinute'] = pd.to_datetime(self.data['tripStartClock'], format='%H:%M').dt.minute
+        self.data['tripEndHour'] = pd.to_datetime(self.data['tripEndClock'], format='%H:%M').dt.hour
+        self.data['tripEndMinute'] = pd.to_datetime(self.data['tripEndClock'], format='%H:%M').dt.minute
+        if weekday:
+            self.addStrColumnFromVariable(colName='weekdayStr', varName='tripStartWeekday')
+        if purpose:
+            self.addStrColumnFromVariable(colName='purposeStr', varName='tripPurpose')
+
+    def convertTypes(self):
+        """
+        Convert raw column types to predefined python types as specified in parseConfig['inputDTypes'][datasetID].
+        This is mainly done for performance reasons. But also in order to avoid index values that are of type int
+        to be cast to float. The function operates only on self.data and writes back changes to self.data
+
+        :return: None
+        """
+
+        # Filter for dataset specific columns
+        conversionDict = self.parseConfig['inputDTypes'][self.datasetID]
+        keys = {iCol for iCol in conversionDict.keys() if iCol in self.data.columns}
+        self.subDict = {key: conversionDict[key] for key in conversionDict.keys() & keys}
+        # German df has commas instead of dots in floats
+        for i, x in enumerate(list(self.data.tripDistance)):
+            self.data.at[i, 'tripDistance'] = x.replace(',', '.')
+        for i, x in enumerate(list(self.data.tripWeight)):
+            self.data.at[i, 'tripWeight'] = x.replace(',', '.')
+        self.data = self.data.astype(self.subDict)
+
+    def updateEndTimestamp(self):
+        """
+        :return:
+        """
+        self.data['tripEndNextDay'] = np.where(self.data['timestampEnd'].dt.day > self.data['timestampStart'].dt.day, 1, 0)
+        endsFollowingDay = self.data['tripEndNextDay'] == 1
+        self.data.loc[endsFollowingDay, 'timestampEnd'] = self.data.loc[endsFollowingDay,
+                                                                            'timestampEnd'] + pd.offsets.Day(1)
+
+
+    def excludeHours(self):
+        """
+        Removes trips where both start and end trip time are missing
+        """
+        self.data = self.data.loc[(self.data['tripStartClock'] != '-1:-1') & (self.data['tripEndClock'] != '-1:-1'), :]
+
+
+    def process(self):
+        """
+        Wrapper function for harmonising and filtering the dataset.
+        """
+        self.selectColumns()
+        self.harmonizeVariables()
+        self.convertTypes()
+        self.excludeHours()
+        self.checkFilterDict()
+        self.filter()
+        self.filterConsistentHours()
+        self.addStrColumns()
+        self.composeStartAndEndTimestamps()
+        self.updateEndTimestamps()
+        self.harmonizeVariablesGenericIdNames()
+        print('Parsing completed')
+
 if __name__ == '__main__':
-    datasetID = 'MiD17'
-    pathLocalPathConfig = Path.cwd().parent / 'config' / 'localPathConfig.yaml'  # pathLib syntax for windows, max, linux compatibility, see https://realpython.com/python-pathlib/ for an intro
-    with open(pathLocalPathConfig) as ipf:
-        localPathConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    pathParseConfig = Path.cwd().parent / 'config' / 'parseConfig.yaml'
-    with open(pathParseConfig) as ipf:
-        parseConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    pathGlobalConfig = Path.cwd().parent / 'config' / 'globalConfig.yaml'
-    with open(pathGlobalConfig) as ipf:
-        globalConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    p = DataParser(localPathConfig=localPathConfig, parseConfig=parseConfig, globalConfig=globalConfig,
-                   loadEncrypted=False, datasetID=datasetID)
-    print(p.data.head())
+    from vencopy.scripts.globalFunctions import loadConfigDict
+    configNames = ('globalConfig', 'localPathConfig', 'parseConfig', 'tripConfig', 'gridConfig', 'flexConfig', 'evaluatorConfig')
+    configDict = loadConfigDict(configNames)
+
+    datasetID = 'MiD17' #options are MiD08, MiD17, KiD
+    #datasetID = 'KiD'
+    vpData = ParseMiD(configDict=configDict, datasetID=datasetID)
+    #vpData = DataParser(configDict=configDict, loadEncrypted=False, datasetID=datasetID)
+    vpData.process()
