@@ -6,18 +6,20 @@ __birthdate__ = '31.12.2019'
 __status__ = 'prod'  # options are: dev, test, prod
 __license__ = 'BSD-3-Clause'
 
+if __package__ is None or __package__ == '':
+    import sys
+    from os import path
+    sys.path.append(path.dirname(path.dirname(path.dirname(__file__))))
 
 import pandas as pd
 import numpy as np
-from typing import Callable
+from typing import Tuple
 from pathlib import Path
-import yaml
-import os
 from vencopy.scripts.globalFunctions import createFileString
 
 
 class TripDiaryBuilder:
-    def __init__(self, tripConfig: dict, globalConfig: dict, ParseData, datasetID: str = 'MiD17', debug: bool=False):
+    def __init__(self, configDict: dict, ParseData, datasetID: str = 'MiD17', debug: bool=False):
         """
         Class to build diaries of trips and parking purposes ("purposes"). Currently a discretization of 1 hour is
         used and tested. Some assumptions are taken in order to build the diaries:
@@ -29,16 +31,17 @@ class TripDiaryBuilder:
         3. If an hour comprises multiple purposes (e.g. driving and home), the purpose with the higher share is taken.
         Potential ambiguities in the diaries are later corrected in the gridModel.
 
-        :param tripConfig: In beta release (0.1.0) an empty file
-        :param globalConfig: Config holding relative paths, run labels and filenames
+        :param configDict: A dictionary containing multiple yaml config files
         :param ParseData: Class instance of ParseData storing MiD data
         :param datasetID: Can be 'MiD17' or 'MiD08' depending on input data set
         :param debug: Boolean argument. If True, only the first 2000 trips are converted to diaries. This parameter is
         used in the beta since the performance of the current tripDiaryBuilder is quite low.
         """
 
-        self.tripConfig = tripConfig
-        self.globalConfig = globalConfig
+        self.tripConfig = configDict['tripConfig']
+        self.globalConfig = configDict['globalConfig']
+        self.localPathConfig = configDict['localPathConfig']
+        self.datasetID = datasetID
         self.parsedData = ParseData
         self.tripDataClean = None
         self.tripDistanceDiary = None
@@ -47,21 +50,23 @@ class TripDiaryBuilder:
             self.tripDataClean = self.calculateConsistentHourlyShares(data=ParseData.data.loc[0:2000, :])
         else:
             self.tripDataClean = self.calculateConsistentHourlyShares(data=ParseData.data)
-        self.tripDistanceDiary = self.tripDistanceAllocation(globalConfig)
+        self.tripDistanceDiary = self.tripDistanceAllocation(globalConfig=self.globalConfig)
         self.tripPurposeAllocation()
-        self.writeOut(globalConfig=globalConfig, datasetID=datasetID, dataDrive=self.tripDistanceDiary,
+        self.writeOut(globalConfig=self.globalConfig, datasetID=datasetID, dataDrive=self.tripDistanceDiary,
                       dataPurpose=self.tripPurposeDiary)
 
     def tripDuration(self, timestampStart: np.datetime64, timestampEnd: np.datetime64) -> np.datetime64:
         return timestampEnd - timestampStart
 
-    def calcHourShareStart(self, timestampStart: pd.Series, timestampEnd: pd.Series, duration: pd.Series) -> (pd.Series,
-                                                                                                           pd.Series):
+    def calcHourShareStart(self, timestampStart: pd.Series, timestampEnd: pd.Series, duration: pd.Series) -> Tuple[pd.Series,
+                                                                                                           pd.Series]:
         """
         :param timestampStart: start time of a trip
         :param timestampEnd:  end time of a trip
         :param duration: duration of a trip
-        :return: Returns a data frame of share of individual trip for trips completed in an hour and more w.r.t start time of the trip
+        :return: Returns a data frame of share of individual trip for trips completed in an hour and more w.r.t start
+                 time of the trip
+
         """
         isSameHourTrip = timestampStart.dt.hour == timestampEnd.dt.hour
         shareSameHour = (timestampEnd.dt.minute - timestampStart.dt.minute) / (duration.dt.seconds / 60)
@@ -75,7 +80,9 @@ class TripDiaryBuilder:
         :param timestampEnd: end time of a trip
         :param duration: duration of a trip
         :param isSameHourTrip: data frame containing same start time of various trips
-        :return: Returns a data frame of share of individual trip for trips completed in an hour and more w.r.t end time of the trip
+        :return: Returns a data frame of share of individual trip for trips completed in an hour and more w.r.t end time
+                 of the trip
+
         """
         share = timestampEnd.dt.minute / (duration.dt.seconds / 60)
         return share.where(~isSameHourTrip, 0)
@@ -106,14 +113,23 @@ class TripDiaryBuilder:
         :return: Returns a data frame of number of full hours of all trips
         """
         timedeltaTrip = timestampEnd - timestampStart
-        numberOfHours = timedeltaTrip.apply(lambda x: x.components.hours)
+        numberOfHours = timestampEnd.dt.hour - (timestampStart.dt.hour + 1)
+        # DEPRECATED
+        #numberOfHours = timedeltaTrip.apply(lambda x: x.components.hours)
         numberOfDays = timedeltaTrip.apply(lambda x: x.components.days)
         minLeftFirstHour = pd.to_timedelta(60 - timestampStart.dt.minute, unit='m')
         hasFullHourAfterFirstHour = (timedeltaTrip - minLeftFirstHour) >= pd.Timedelta(1, unit='h')
+        # NEW FIX FOR OVERNIGHT / FIRST HOUR = FULL HOUR
+        # Count down the full hours where first hour is full hour AND the trip has at least one other full hour
+        # countDown = ((minLeftFirstHour == pd.to_timedelta(60, unit='m')) & hasFullHourAfterFirstHour)
+
+        numberOfHours = numberOfHours.where(numberOfHours > 0, other=0)
         numberOfHours = numberOfHours.where(hasFullHourAfterFirstHour, other=0)
+        # numberOfHours = numberOfHours.where(~countDown, other=numberOfHours-1)
         return numberOfHours.where(numberOfDays != -1, other=0)
 
-    def calcFullHourTripLength(self, duration: pd.Series, numberOfFullHours: pd.Series, tripLength: pd.Series) -> \
+    def calcFullHourTripLength(self, duration: pd.Series, numberOfFullHours: pd.Series, tripLength: pd.Series,
+                               shareStartHour:pd.Series, shareEndHour: pd.Series) -> \
             pd.Series:
         """
         Calculates the share of the full trip hours. E.g. the fullHourTripLength of a trip starting at 1:45 and ending
@@ -125,9 +141,11 @@ class TripDiaryBuilder:
         :return: Returns a Series of full hour trip lengths for all trips
         """
 
-        fullHourTripLength = (numberOfFullHours / (duration.dt.seconds / 3600)) * tripLength
+        # fullHourTripLength = (numberOfFullHours / (duration.dt.seconds / 3600)) * tripLength
+        fullHourTripShare = 1 - (shareStartHour + shareEndHour)
+        fullHourTripLength = fullHourTripShare * tripLength
         fullHourTripLength.loc[duration == pd.Timedelta(0)] = 0  # set trip length to 0 that would otherwise be NaN
-        return fullHourTripLength
+        return fullHourTripShare, fullHourTripLength
 
     def calcHourlyShares(self, data: pd.DataFrame, ts_st: str, ts_en: str) -> pd.DataFrame:
         """
@@ -136,14 +154,18 @@ class TripDiaryBuilder:
         :param data: Trip data
         :param ts_st: String specifying the trip start column
         :param ts_en: String specifying the trip end column
-        :return: data frame consisting additional information regarding share of a trip, number of full hours and lenght of each trip
+        :return: data frame consisting additional information regarding share of a trip, number of full hours and
+                 lenght of each trip
         """
 
         duration = self.tripDuration(data.loc[:, ts_st], data.loc[:, ts_en])
-        data.loc[:, 'shareStartHour'], data.loc[:, 'shareEndHour'] = self.calcDistanceShares(data, duration, ts_st, ts_en)
-        data.loc[:, 'noOfFullHours'] = self.numberOfFullHours(data.loc[:, ts_st], data.loc[:, ts_en])
-        data.loc[:, 'fullHourTripLength'] = self.calcFullHourTripLength(duration, data.loc[:, 'noOfFullHours'],
-                                                                   data.loc[:, 'tripDistance'])
+        data.loc[:, 'shareStartHour'], data.loc[:, 'shareEndHour'] = self.calcDistanceShares(data, duration, ts_st,
+                                                                                             ts_en)
+        data.loc[:, 'noOfFullHours'] = self.numberOfFullHours(timestampStart=data.loc[:, ts_st],
+                                                              timestampEnd=data.loc[:, ts_en])
+        data.loc[:, 'fullHourTripShare'], data.loc[:, 'fullHourTripLength'] = \
+            self.calcFullHourTripLength(duration, data.loc[:, 'noOfFullHours'], data.loc[:, 'tripDistance'],
+                                        data.loc[:, 'shareStartHour'], data.loc[:, 'shareEndHour'])
         return data
 
     def calculateConsistentHourlyShares(self, data: pd.DataFrame):
@@ -156,14 +178,16 @@ class TripDiaryBuilder:
 
         print('Calculating hourly shares')
         if not data._is_view:
-            data = data.copy()  #FIXME: why is data._is_view False if we get a view
+            data = data.copy()  # FIXME: why is data._is_view False if we get a view
         tripDataWHourlyShares = self.calcHourlyShares(data, ts_st='timestampStart', ts_en='timestampEnd')
 
         # Filter out implausible hourly share combinations
         return tripDataWHourlyShares.loc[~((tripDataWHourlyShares['shareStartHour'] != 1) &
-                                                       (tripDataWHourlyShares['shareEndHour'] == 0) &
-                                                       (tripDataWHourlyShares['noOfFullHours'] == 0)), :]
+                                           (tripDataWHourlyShares['shareEndHour'] == 0) &
+                                           (tripDataWHourlyShares['noOfFullHours'] == 0)), :]
 
+
+    # DEPRECATED WILL BE REMOVED IN NEXT PATCH due to one-liner
     def initiateHourDataframe(self, indexCol, nHours: int) -> pd.DataFrame:
         """
         Sets up an empty dataframe to be filled with hourly data.
@@ -174,6 +198,7 @@ class TripDiaryBuilder:
         """
         return pd.DataFrame(index=indexCol, columns=range(nHours))
 
+    # DEPRECATED WILL BE REMOVED IN NEXT PATCH due to one-liner
     def fillDataframe(self, hourlyArray: pd.DataFrame, fillFunction) -> pd.DataFrame:
         return hourlyArray.apply(fillFunction, axis=1)
 
@@ -184,24 +209,31 @@ class TripDiaryBuilder:
         :param tripData: Input trip data with hourly distances of all hourly trips
         :return: Merged trip distance diaries
         """
-        dataDay = tripData.groupby(['hhPersonID']).sum()
+        dataDay = tripData.groupby(['genericID']).sum()
         dataDay = dataDay.drop('tripID', axis=1)
         return dataDay
 
     def initiateColRange(self, row: pd.Series):
         """
-        Returns a range object with start and end hour as limits
+        Returns a range object with start and end hour as limits describing only full hours e.g.
+        for a trip from 18:30 to 21:20 the hours 19 and 20 (21 will be included in the range but disregarded
+        in the following)
 
         :param row: A trip observation
-        :return:
+        :return: range of full hours
         """
+        nHours = self.globalConfig['numberOfHours']
         if row['tripStartHour'] + 1 < row['tripEndHour']:
-            return range(row['tripStartHour'] + 1, row['tripEndHour'])  # The hour of arrival (tripEndHour) will
-            # not be indexed further below but is part of the range() object
+            return range(row['tripStartHour'] + 1, row['tripEndHour'])
+            # The hour of arrival (tripEndHour) will not be indexed further below but is part of the range() object
+        elif row['tripEndNextDay'] and (row['tripStartHour'] <= nHours-1 or row['tripStartMinute'] == 0):
+            lst = [iC for iC in range(0, row['tripEndHour'])]
+            return lst + [iC for iC in range(row['tripStartHour'] + 1, nHours)]
+
         else:
             return None
 
-    def tripDistanceAllocation(self, globalConfig : dict) -> pd.DataFrame:
+    def tripDistanceAllocation(self, globalConfig: dict) -> pd.DataFrame:
         """
         Wrapper function for the conversion of trip distance values (in a column) to hourly trip distance diaries.
 
@@ -209,12 +241,16 @@ class TripDiaryBuilder:
         :return: Trip distance diary as a pd.DataFrame
         """
         print('Trip distance diary setup starting')
-        self.formatDF = self.initiateHourDataframe(indexCol=self.tripDataClean.index, nHours=globalConfig['numberOfHours'])
+        self.formatDF = pd.DataFrame(index=self.tripDataClean.index, columns=range(globalConfig['numberOfHours']))
         fillHourValues = FillHourValues(data=self.tripDataClean, rangeFunction=self.initiateColRange)
-        driveDataTrips = self.fillDataframe(self.formatDF, fillFunction=fillHourValues)
-        driveDataTrips.loc[:, ['hhPersonID', 'tripID']] = pd.DataFrame(self.tripDataClean.loc[:, ['hhPersonID',
-                                                                                                  'tripID']])
-        driveDataTrips = driveDataTrips.astype({'hhPersonID': int, 'tripID': int})
+        driveDataTrips = self.formatDF.apply(fillHourValues, axis=1)
+
+        # DEPRECATED
+        # driveDataTrips = self.fillDataframe(self.formatDF, fillFunction=fillHourValues)
+
+        driveDataTrips.loc[:, ['genericID', 'tripID']] = pd.DataFrame(self.tripDataClean.loc[:, ['genericID',
+                                                                                                 'tripID']])
+        driveDataTrips = driveDataTrips.astype({'genericID': int, 'tripID': int})
         print('Finished trip distance diary setup')
         return self.mergeTrips(driveDataTrips)
 
@@ -250,25 +286,26 @@ class TripDiaryBuilder:
                 startHour = arrival.hour + 1  # Cases 2a and b
         return startHour
 
-    def fillDayPurposes(self, tripData: pd.DataFrame, purposeDataDays: pd.DataFrame) -> pd.DataFrame:  # FixMe: Ask Ben for performance improvements
+    def fillDayPurposes(self, tripData: pd.DataFrame, purposeDataDays: pd.DataFrame) -> pd.DataFrame:
+        # FixMe: Ask Ben for performance improvements
         """
         Main purpose diary builder function. Root of low performance of tripDiaryBuilder. Will be improved in future
         releases.
 
         :param tripData: data frame holding all the information about individual trip
         :param purposeDataDays: DataFrame with 24 (hour) columns holding 0s and 'DRIVING' for trip hours (for hours
-        where majority of time is driving)
+               where majority of time is driving)
         :return: Returns a data frame of individual trip with it's hourly activity or parking purpose
         """
         hpID = str()
         maxWID = int()
         maxHour = len(purposeDataDays.columns)
 
-        # # uniques = tripData['hhPersonID'].unique()
+        # # uniques = tripData['genericID'].unique()
         #
-        # for iSubData in tripData.groupby('hhPersonID'):
+        # for iSubData in tripData.groupby('genericID'):
         #
-        #     currentPerson = tripData['hhPersonID'] == hpID
+        #     currentPerson = tripData['genericID'] == hpID
         #     allWIDs = tripData.loc[currentPerson, 'tripID']  # FIXME perf
         #     minWID = allWIDs.min()  # FIXME perf
         #     maxWID = allWIDs.max()  # FIXME perf
@@ -288,10 +325,10 @@ class TripDiaryBuilder:
         # Solution 1: use enumerate in order to get rowNumber instead of index and then .iloc below
         # Solution 2: Rename columns
         for idx, iRow in tripData.iterrows():
-            isSameHPID = hpID == iRow['hhPersonID']
+            isSameHPID = hpID == iRow['genericID']
             if not isSameHPID:
-                hpID = iRow['hhPersonID']
-                allWIDs = tripData.loc[tripData['hhPersonID'] == hpID, 'tripID']  # FIXME perf
+                hpID = iRow['genericID']
+                allWIDs = tripData.loc[tripData['genericID'] == hpID, 'tripID']  # FIXME perf
                 minWID = allWIDs.min()  # FIXME perf
                 maxWID = allWIDs.max()  # FIXME perf
 
@@ -316,7 +353,7 @@ class TripDiaryBuilder:
                     purposeDataDays.loc[hpID, range(iRow['tripEndHour'] + 1, maxHour)] = 'HOME'
             else:
                 purposeHourStart = self.determinePurposeStartHour(tripData.loc[idxOld, 'timestampStart'],
-                                                             tripData.loc[idxOld, 'timestampEnd'])  # FIXME perf?
+                                                                  tripData.loc[idxOld, 'timestampEnd'])  # FIXME perf?
                 if iRow['timestampStart'].minute <= 30:
                     hoursBetween = range(purposeHourStart,
                                          iRow['tripStartHour'])  # FIXME: case differentiation on arrival hour
@@ -334,7 +371,8 @@ class TripDiaryBuilder:
                 # else:
                 #     # hoursBetween = range(purposeHourStart,
                 #     #                      iRow['tripStartHour'] + 1)
-                #     purposeDataDays.loc[hpID, purposeHourStart:iRow['tripStartHour'] + 1] = tripData.loc[idxOld, 'purposeStr']
+                #     purposeDataDays.loc[hpID, purposeHourStart:iRow['tripStartHour'] + 1] = tripData.loc[idxOld,
+                #     'purposeStr']
                 if iRow['tripID'] == maxWID:
                     if iRow['timestampEnd'].minute <= 30:
                         purposeDataDays.loc[hpID, range(iRow['tripEndHour'], maxHour)] = 'HOME'
@@ -356,16 +394,16 @@ class TripDiaryBuilder:
         print('Finished purpose replacements')
         print(f'There are {len(self.tripPurposeDiary)} daily trip diaries.')
 
-    # improved purpose allocation approach
-    def mapHHPIDToTripID(self, tripData):
-        idCols = self.tripDataClean.loc[:, ['hhPersonID', 'tripID']]
-        idCols.loc['nextTripID'] = idCols['tripID'].shift(-1, fill_value=0)
-        tripDict = dict.fromkeys(set(idCols['hhPersonID']))
-        for ihhpID in tripDict.keys():
-            tripDict[ihhpID] = set(idCols.loc[idCols['hhPersonID'] == ihhpID, 'tripID'])
-        return tripDict
+    # # improved purpose allocation approach
+    # def mapHHPIDToTripID(self, tripData):
+    #     idCols = self.tripDataClean.loc[:, ['genericID', 'tripID']]
+    #     idCols.loc['nextTripID'] = idCols['tripID'].shift(-1, fill_value=0)
+    #     tripDict = dict.fromkeys(set(idCols['genericID']))
+    #     for ihhpID in tripDict.keys():
+    #         tripDict[ihhpID] = set(idCols.loc[idCols['genericID'] == ihhpID, 'tripID'])
+    #     return tripDict
 
-    def writeOut(self, globalConfig:dict, dataDrive: pd.DataFrame, dataPurpose: pd.DataFrame, datasetID: str = 'MiD17'):
+    def writeOut(self, globalConfig: dict, dataDrive: pd.DataFrame, dataPurpose: pd.DataFrame, datasetID: str = 'MiD17'):
         """
         General writeout utility for tripDiaries
 
@@ -375,12 +413,13 @@ class TripDiaryBuilder:
         :param datasetID: ID used for filenames
         :return: None
         """
-        dataDrive.to_csv(Path(globalConfig['pathRelative']['diaryOutput']) /
+
+        dataDrive.to_csv(Path(self.localPathConfig['pathAbsolute']['vencoPyRoot']) / globalConfig['pathRelative']['diaryOutput'] /
                          createFileString(globalConfig=globalConfig, fileKey='inputDataDriveProfiles',
-                                          datasetID=datasetID),
-                         na_rep=0)
-        dataPurpose.to_csv(Path(globalConfig['pathRelative']['diaryOutput']) /
-                          createFileString(globalConfig=globalConfig, fileKey='purposesProcessed', datasetID=datasetID))
+                                          datasetID=datasetID), na_rep='0')
+        dataPurpose.to_csv(Path(self.localPathConfig['pathAbsolute']['vencoPyRoot']) / globalConfig['pathRelative']['diaryOutput'] /
+                           createFileString(globalConfig=globalConfig, fileKey='purposesProcessed',
+                                            datasetID=datasetID))
         print(f"Drive data and trip purposes written to files "
               f"{createFileString(globalConfig=globalConfig, fileKey='inputDataDriveProfiles', datasetID=datasetID)} "
               f"and {createFileString(globalConfig=globalConfig, fileKey='purposesProcessed', datasetID=datasetID)}")
@@ -393,33 +432,32 @@ class FillHourValues:
         self.endHour = data['tripEndHour']
         self.distanceEndHour = data['shareEndHour'] * data['tripDistance']
         self.fullHourCols = data.apply(rangeFunction, axis=1)
+        #self.fullHourRange = data['fullHourTripLength'] / data['noOfFullHours']
         self.fullHourRange = data['fullHourTripLength']
 
     def __call__(self, row):
         idx = row.name
+        # if np.isnan(row[self.startHour[idx]]):
         row[self.startHour[idx]] = self.distanceStartHour[idx]
+        # else:
+        #    row[self.startHour[idx]] = row[self.startHour[idx]] + self.distanceStartHour[idx]
+
         if self.endHour[idx] != self.startHour[idx]:
             row[self.endHour[idx]] = self.distanceEndHour[idx]
-        if isinstance(self.fullHourCols[idx], range):
-            row[self.fullHourCols[idx]] = self.fullHourRange[idx]
+        if isinstance(self.fullHourCols[idx], range) or isinstance(self.fullHourCols[idx], list):
+            row[self.fullHourCols[idx]] = self.fullHourRange[idx] / len(self.fullHourCols[idx])
         return row
 
 
 if __name__ == '__main__':
+    datasetID = 'MiD17' #options are MiD08, MiD17, KiD
+    # datasetID = 'KiD'
+
     from vencopy.classes.dataParsers import DataParser
-    pathGlobalConfig = Path.cwd().parent / 'config' / 'globalConfig.yaml'  # pathLib syntax for windows, max, linux compatibility, see https://realpython.com/python-pathlib/ for an intro
-    with open(pathGlobalConfig) as ipf:
-        globalConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    pathParseConfig = Path.cwd().parent / 'config' / 'parseConfig.yaml'
-    with open(pathParseConfig) as ipf:
-        parseConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    pathTripConfig = Path.cwd().parent / 'config' / 'tripConfig.yaml'
-    with open(pathTripConfig) as ipf:
-        tripConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    pathLocalPathConfig = Path.cwd().parent / 'config' / 'localPathConfig.yaml'
-    with open(pathLocalPathConfig) as ipf:
-        localPathConfig = yaml.load(ipf, Loader=yaml.SafeLoader)
-    os.chdir(localPathConfig['pathAbsolute']['vencoPyRoot'])
-    vpData = DataParser(parseConfig=parseConfig, globalConfig=globalConfig, localPathConfig=localPathConfig,
-                        loadEncrypted=False)
-    vpDiary = TripDiaryBuilder(tripConfig=tripConfig, globalConfig=globalConfig, ParseData=vpData)
\ No newline at end of file
+    from vencopy.scripts.globalFunctions import loadConfigDict
+
+    configNames = ('globalConfig', 'localPathConfig', 'parseConfig', 'tripConfig', 'gridConfig', 'flexConfig', 'evaluatorConfig')
+    configDict = loadConfigDict(configNames)
+    vpData = DataParser(configDict=configDict, loadEncrypted=False, datasetID=datasetID)
+    vpData.process()
+    vpDiary = TripDiaryBuilder(configDict=configDict, ParseData=vpData, datasetID=datasetID, debug=True)
